package javax.swing.tree;

import java.util.NoSuchElementException;
import sun.swing.SwingUtilities2;
import javax.swing.event.TreeModelEvent;
import java.util.Enumeration;
import java.util.Stack;
import java.util.Hashtable;
import java.awt.Rectangle;
import java.util.Vector;

public class VariableHeightLayoutCache extends AbstractLayoutCache
{
    private Vector<Object> visibleNodes;
    private boolean updateNodeSizes;
    private VariableHeightLayoutCache.TreeStateNode root;
    private Rectangle boundsBuffer;
    private Hashtable<TreePath, VariableHeightLayoutCache.TreeStateNode> treePathMapping;
    private Stack<Stack<TreePath>> tempStacks;
    
    public VariableHeightLayoutCache() {
        this.tempStacks = new Stack();
        this.visibleNodes = new Vector();
        this.boundsBuffer = new Rectangle();
        this.treePathMapping = new Hashtable();
    }
    
    @Override
    public void setModel(TreeModel model) {
        super.setModel(model);
        this.rebuild(false);
    }
    
    @Override
    public void setRootVisible(boolean rootVisible) {
        if (this.isRootVisible() != rootVisible && this.root != null) {
            if (rootVisible) {
                this.root.updatePreferredSize(0);
                this.visibleNodes.insertElementAt(this.root, 0);
            }
            else if (this.visibleNodes.size() > 0) {
                this.visibleNodes.removeElementAt(0);
                if (this.treeSelectionModel != null) {
                    this.treeSelectionModel.removeSelectionPath(this.root.getTreePath());
                }
            }
            if (this.treeSelectionModel != null) {
                this.treeSelectionModel.resetRowSelection();
            }
            if (this.getRowCount() > 0) {
                this.getNode(0).setYOrigin(0);
            }
            this.updateYLocationsFrom(0);
            this.visibleNodesChanged();
        }
        super.setRootVisible(rootVisible);
    }
    
    @Override
    public void setRowHeight(int rowHeight) {
        if (rowHeight != this.getRowHeight()) {
            super.setRowHeight(rowHeight);
            this.invalidateSizes();
            this.visibleNodesChanged();
        }
    }
    
    @Override
    public void setNodeDimensions(AbstractLayoutCache.NodeDimensions nodeDimensions) {
        super.setNodeDimensions(nodeDimensions);
        this.invalidateSizes();
        this.visibleNodesChanged();
    }
    
    @Override
    public void setExpandedState(TreePath treePath, boolean b) {
        if (treePath != null) {
            if (b) {
                this.ensurePathIsExpanded(treePath, true);
            }
            else {
                VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, false, true);
                if (nodeForPath != null) {
                    nodeForPath.makeVisible();
                    nodeForPath.collapse();
                }
            }
        }
    }
    
    @Override
    public boolean getExpandedState(TreePath treePath) {
        VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        return nodeForPath != null && (nodeForPath.isVisible() && nodeForPath.isExpanded());
    }
    
    @Override
    public Rectangle getBounds(TreePath treePath, Rectangle rectangle) {
        VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath != null) {
            if (this.updateNodeSizes) {
                this.updateNodeSizes(false);
            }
            return nodeForPath.getNodeBounds(rectangle);
        }
        else {
            return null;
        }
    }
    
    @Override
    public TreePath getPathForRow(int n) {
        if (n >= 0 && n < this.getRowCount()) {
            return this.getNode(n).getTreePath();
        }
        return null;
    }
    
    @Override
    public int getRowForPath(TreePath treePath) {
        if (treePath == null) {
            return -1;
        }
        VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath != null) {
            return nodeForPath.getRow();
        }
        return -1;
    }
    
    @Override
    public int getRowCount() {
        return this.visibleNodes.size();
    }
    
    @Override
    public void invalidatePathBounds(TreePath treePath) {
        VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath != null) {
            nodeForPath.markSizeInvalid();
            if (nodeForPath.isVisible()) {
                this.updateYLocationsFrom(nodeForPath.getRow());
            }
        }
    }
    
    @Override
    public int getPreferredHeight() {
        int rowCount = this.getRowCount();
        if (rowCount > 0) {
            VariableHeightLayoutCache.TreeStateNode node = this.getNode(rowCount - 1);
            return node.getYOrigin() + node.getPreferredHeight();
        }
        return 0;
    }
    
    @Override
    public int getPreferredWidth(Rectangle rectangle) {
        if (this.updateNodeSizes) {
            this.updateNodeSizes(false);
        }
        return this.getMaxNodeWidth();
    }
    
    @Override
    public TreePath getPathClosestTo(int n, int n2) {
        if (this.getRowCount() == 0) {
            return null;
        }
        if (this.updateNodeSizes) {
            this.updateNodeSizes(false);
        }
        return this.getNode(this.getRowContainingYLocation(n2)).getTreePath();
    }
    
    @Override
    public Enumeration<TreePath> getVisiblePathsFrom(TreePath treePath) {
        VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath != null) {
            return new VariableHeightLayoutCache.VisibleTreeStateNodeEnumeration(nodeForPath);
        }
        return null;
    }
    
    @Override
    public int getVisibleChildCount(TreePath treePath) {
        VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        return (nodeForPath != null) ? nodeForPath.getVisibleChildCount() : 0;
    }
    
    @Override
    public void invalidateSizes() {
        if (this.root != null) {
            this.root.deepMarkSizeInvalid();
        }
        if (!this.isFixedRowHeight() && this.visibleNodes.size() > 0) {
            this.updateNodeSizes(true);
        }
    }
    
    @Override
    public boolean isExpanded(TreePath treePath) {
        if (treePath != null) {
            VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
            return nodeForPath != null && nodeForPath.isExpanded();
        }
        return false;
    }
    
    @Override
    public void treeNodesChanged(TreeModelEvent event) {
        if (event != null) {
            int[] childIndices = event.getChildIndices();
            VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(SwingUtilities2.getTreePath(event, this.getModel()), false, false);
            if (nodeForPath != null) {
                Object value = nodeForPath.getValue();
                nodeForPath.updatePreferredSize();
                if (nodeForPath.hasBeenExpanded() && childIndices != null) {
                    for (int i = 0; i < childIndices.length; ++i) {
                        VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)nodeForPath.getChildAt(childIndices[i]);
                        treeStateNode.setUserObject(this.treeModel.getChild(value, childIndices[i]));
                        treeStateNode.updatePreferredSize();
                    }
                }
                else if (nodeForPath == this.root) {
                    nodeForPath.updatePreferredSize();
                }
                if (!this.isFixedRowHeight()) {
                    int row = nodeForPath.getRow();
                    if (row != -1) {
                        this.updateYLocationsFrom(row);
                    }
                }
                this.visibleNodesChanged();
            }
        }
    }
    
    @Override
    public void treeNodesInserted(TreeModelEvent event) {
        if (event != null) {
            int[] childIndices = event.getChildIndices();
            VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(SwingUtilities2.getTreePath(event, this.getModel()), false, false);
            if (nodeForPath != null && childIndices != null && childIndices.length > 0) {
                if (nodeForPath.hasBeenExpanded()) {
                    int childCount = nodeForPath.getChildCount();
                    nodeForPath.getValue();
                    int n = (nodeForPath == this.root && !this.rootVisible) || (nodeForPath.getRow() != -1 && nodeForPath.isExpanded());
                    for (int i = 0; i < childIndices.length; ++i) {
                        this.createNodeAt(nodeForPath, childIndices[i]);
                    }
                    if (childCount == 0) {
                        nodeForPath.updatePreferredSize();
                    }
                    if (this.treeSelectionModel != null) {
                        this.treeSelectionModel.resetRowSelection();
                    }
                    if (!this.isFixedRowHeight() && (n || (childCount == 0 && nodeForPath.isVisible()))) {
                        if (nodeForPath == this.root) {
                            this.updateYLocationsFrom(0);
                        }
                        else {
                            this.updateYLocationsFrom(nodeForPath.getRow());
                        }
                        this.visibleNodesChanged();
                    }
                    else if (!(n == 0)) {
                        this.visibleNodesChanged();
                    }
                }
                else if (this.treeModel.getChildCount(nodeForPath.getValue()) - childIndices.length == 0) {
                    nodeForPath.updatePreferredSize();
                    if (!this.isFixedRowHeight() && nodeForPath.isVisible()) {
                        this.updateYLocationsFrom(nodeForPath.getRow());
                    }
                }
            }
        }
    }
    
    @Override
    public void treeNodesRemoved(TreeModelEvent event) {
        if (event != null) {
            int[] childIndices = event.getChildIndices();
            VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(SwingUtilities2.getTreePath(event, this.getModel()), false, false);
            if (nodeForPath != null && childIndices != null && childIndices.length > 0) {
                if (nodeForPath.hasBeenExpanded()) {
                    int n = (nodeForPath == this.root && !this.rootVisible) || (nodeForPath.getRow() != -1 && nodeForPath.isExpanded());
                    for (int i = childIndices.length - 1; i >= 0; --i) {
                        VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)nodeForPath.getChildAt(childIndices[i]);
                        if (treeStateNode.isExpanded()) {
                            treeStateNode.collapse(false);
                        }
                        if (!(n == 0)) {
                            int row = treeStateNode.getRow();
                            if (row != -1) {
                                this.visibleNodes.removeElementAt(row);
                            }
                        }
                        nodeForPath.remove(childIndices[i]);
                    }
                    if (nodeForPath.getChildCount() == 0) {
                        nodeForPath.updatePreferredSize();
                        if (nodeForPath.isExpanded() && nodeForPath.isLeaf()) {
                            nodeForPath.collapse(false);
                        }
                    }
                    if (this.treeSelectionModel != null) {
                        this.treeSelectionModel.resetRowSelection();
                    }
                    if (!this.isFixedRowHeight() && (n || (nodeForPath.getChildCount() == 0 && nodeForPath.isVisible()))) {
                        if (nodeForPath == this.root) {
                            if (this.getRowCount() > 0) {
                                this.getNode(0).setYOrigin(0);
                            }
                            this.updateYLocationsFrom(0);
                        }
                        else {
                            this.updateYLocationsFrom(nodeForPath.getRow());
                        }
                        this.visibleNodesChanged();
                    }
                    else if (!(n == 0)) {
                        this.visibleNodesChanged();
                    }
                }
                else if (this.treeModel.getChildCount(nodeForPath.getValue()) == 0) {
                    nodeForPath.updatePreferredSize();
                    if (!this.isFixedRowHeight() && nodeForPath.isVisible()) {
                        this.updateYLocationsFrom(nodeForPath.getRow());
                    }
                }
            }
        }
    }
    
    @Override
    public void treeStructureChanged(TreeModelEvent event) {
        if (event != null) {
            TreePath treePath = SwingUtilities2.getTreePath(event, this.getModel());
            VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(treePath, false, false);
            if (nodeForPath == this.root || (nodeForPath == null && ((treePath == null && this.treeModel != null && this.treeModel.getRoot() == null) || (treePath != null && treePath.getPathCount() == 1)))) {
                this.rebuild(true);
            }
            else if (nodeForPath != null) {
                int expanded = nodeForPath.isExpanded();
                int n = nodeForPath.getRow() != -1;
                VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)nodeForPath.getParent();
                int index = treeStateNode.getIndex(nodeForPath);
                if (n && expanded) {
                    nodeForPath.collapse(false);
                }
                if (!(n == 0)) {
                    this.visibleNodes.removeElement(nodeForPath);
                }
                nodeForPath.removeFromParent();
                this.createNodeAt(treeStateNode, index);
                VariableHeightLayoutCache.TreeStateNode treeStateNode2 = (VariableHeightLayoutCache.TreeStateNode)treeStateNode.getChildAt(index);
                if (n && expanded) {
                    treeStateNode2.expand(false);
                }
                int row = treeStateNode2.getRow();
                if (!this.isFixedRowHeight() && n) {
                    if (row == 0) {
                        this.updateYLocationsFrom(row);
                    }
                    else {
                        this.updateYLocationsFrom(row - 1);
                    }
                    this.visibleNodesChanged();
                }
                else if (!(n == 0)) {
                    this.visibleNodesChanged();
                }
            }
        }
    }
    
    private void visibleNodesChanged() {
    }
    
    private void addMapping(VariableHeightLayoutCache.TreeStateNode value) {
        this.treePathMapping.put(value.getTreePath(), value);
    }
    
    private void removeMapping(VariableHeightLayoutCache.TreeStateNode treeStateNode) {
        this.treePathMapping.remove(treeStateNode.getTreePath());
    }
    
    private VariableHeightLayoutCache.TreeStateNode getMapping(TreePath key) {
        return (VariableHeightLayoutCache.TreeStateNode)this.treePathMapping.get(key);
    }
    
    private Rectangle getBounds(int n, Rectangle rectangle) {
        if (this.updateNodeSizes) {
            this.updateNodeSizes(false);
        }
        if (n >= 0 && n < this.getRowCount()) {
            return this.getNode(n).getNodeBounds(rectangle);
        }
        return null;
    }
    
    private void rebuild(boolean b) {
        this.treePathMapping.clear();
        Object root;
        if (this.treeModel != null && (root = this.treeModel.getRoot()) != null) {
            this.root = this.createNodeForValue(root);
            this.root.path = new TreePath(root);
            this.addMapping(this.root);
            this.root.updatePreferredSize(0);
            this.visibleNodes.removeAllElements();
            if (this.isRootVisible()) {
                this.visibleNodes.addElement(this.root);
            }
            if (!this.root.isExpanded()) {
                this.root.expand();
            }
            else {
                Enumeration<Object> children = this.root.children();
                while (children.hasMoreElements()) {
                    this.visibleNodes.addElement(children.nextElement());
                }
                if (!this.isFixedRowHeight()) {
                    this.updateYLocationsFrom(0);
                }
            }
        }
        else {
            this.visibleNodes.removeAllElements();
            this.root = null;
        }
        if (b && this.treeSelectionModel != null) {
            this.treeSelectionModel.clearSelection();
        }
        this.visibleNodesChanged();
    }
    
    private VariableHeightLayoutCache.TreeStateNode createNodeAt(VariableHeightLayoutCache.TreeStateNode treeStateNode, int n) {
        VariableHeightLayoutCache.TreeStateNode nodeForValue = this.createNodeForValue(this.treeModel.getChild(treeStateNode.getValue(), n));
        treeStateNode.insert(nodeForValue, n);
        nodeForValue.updatePreferredSize(-1);
        int n2 = treeStateNode == this.root;
        if (nodeForValue != null && treeStateNode.isExpanded() && (treeStateNode.getRow() != -1 || n2)) {
            int index;
            if (n == 0) {
                if (n2 && !this.isRootVisible()) {
                    index = 0;
                }
                else {
                    index = treeStateNode.getRow() + 1;
                }
            }
            else if (n == treeStateNode.getChildCount()) {
                index = treeStateNode.getLastVisibleNode().getRow() + 1;
            }
            else {
                index = ((VariableHeightLayoutCache.TreeStateNode)treeStateNode.getChildAt(n - 1)).getLastVisibleNode().getRow() + 1;
            }
            this.visibleNodes.insertElementAt(nodeForValue, index);
            return nodeForValue;
        }
        else {
            return nodeForValue;
        }
    }
    
    private VariableHeightLayoutCache.TreeStateNode getNodeForPath(TreePath treePath, boolean b, boolean b2) {
        if (treePath != null) {
            VariableHeightLayoutCache.TreeStateNode mapping = this.getMapping(treePath);
            if (mapping != null) {
                if (b && !mapping.isVisible()) {
                    return null;
                }
                return mapping;
            }
            else {
                Stack<TreePath> item;
                if (this.tempStacks.size() == 0) {
                    item = new Stack();
                }
                else {
                    item = (Stack)this.tempStacks.pop();
                }
                try {
                    item.push(treePath);
                    treePath = treePath.getParentPath();
                    while (treePath != null) {
                        VariableHeightLayoutCache.TreeStateNode mapping2 = this.getMapping(treePath);
                        if (mapping2 != null) {
                            while (mapping2 != null && item.size() > 0) {
                                treePath = (TreePath)item.pop();
                                mapping2.getLoadedChildren(b2);
                                int indexOfChild = this.treeModel.getIndexOfChild(mapping2.getUserObject(), treePath.getLastPathComponent());
                                if (indexOfChild == -1 || indexOfChild >= mapping2.getChildCount() || (b && !mapping2.isVisible())) {
                                    mapping2 = null;
                                }
                                else {
                                    mapping2 = (VariableHeightLayoutCache.TreeStateNode)mapping2.getChildAt(indexOfChild);
                                }
                            }
                            return mapping2;
                        }
                        item.push(treePath);
                        treePath = treePath.getParentPath();
                    }
                }
                finally {
                    item.removeAllElements();
                    this.tempStacks.push(item);
                }
            }
        }
        return null;
    }
    
    private void updateYLocationsFrom(int n) {
        if (n >= 0 && n < this.getRowCount()) {
            VariableHeightLayoutCache.TreeStateNode node = this.getNode(n);
            VariableHeightLayoutCache.TreeStateNode treeStateNode;
            for (int yOrigin = node.getYOrigin() + node.getPreferredHeight(), i = n + 1; i < this.visibleNodes.size(); yOrigin += treeStateNode.getPreferredHeight(), ++i) {
                treeStateNode = (VariableHeightLayoutCache.TreeStateNode)this.visibleNodes.elementAt(i);
                treeStateNode.setYOrigin(yOrigin);
            }
        }
    }
    
    private void updateNodeSizes(boolean b) {
        this.updateNodeSizes = false;
        int yOrigin;
        VariableHeightLayoutCache.TreeStateNode treeStateNode;
        for (int i = yOrigin = 0; i < this.visibleNodes.size(); yOrigin += treeStateNode.getPreferredHeight(), ++i) {
            treeStateNode = (VariableHeightLayoutCache.TreeStateNode)this.visibleNodes.elementAt(i);
            treeStateNode.setYOrigin(yOrigin);
            if (b || !treeStateNode.hasValidSize()) {
                treeStateNode.updatePreferredSize(i);
            }
        }
    }
    
    private int getRowContainingYLocation(int n) {
        if (this.isFixedRowHeight()) {
            if (this.getRowCount() == 0) {
                return -1;
            }
            return Math.max(0, Math.min(this.getRowCount() - 1, n / this.getRowHeight()));
        }
        else {
            int rowCount;
            if ((rowCount = this.getRowCount()) <= 0) {
                return -1;
            }
            int index;
            int i = index = 0;
            while (i < rowCount) {
                index = (rowCount - i) / 2 + i;
                VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)this.visibleNodes.elementAt(index);
                int yOrigin = treeStateNode.getYOrigin();
                int n2 = yOrigin + treeStateNode.getPreferredHeight();
                if (n < yOrigin) {
                    rowCount = index - 1;
                }
                else if (n >= n2) {
                    i = index + 1;
                }
                else {
                    break;
                }
            }
            if (i == rowCount) {
                index = i;
                if (index >= this.getRowCount()) {
                    index = this.getRowCount() - 1;
                }
            }
            return index;
        }
    }
    
    private void ensurePathIsExpanded(TreePath parentPath, boolean b) {
        if (parentPath != null) {
            if (this.treeModel.isLeaf(parentPath.getLastPathComponent())) {
                parentPath = parentPath.getParentPath();
                b = true;
            }
            if (parentPath != null) {
                VariableHeightLayoutCache.TreeStateNode nodeForPath = this.getNodeForPath(parentPath, false, true);
                if (nodeForPath != null) {
                    nodeForPath.makeVisible();
                    if (b) {
                        nodeForPath.expand();
                    }
                }
            }
        }
    }
    
    private VariableHeightLayoutCache.TreeStateNode getNode(int index) {
        return (VariableHeightLayoutCache.TreeStateNode)this.visibleNodes.elementAt(index);
    }
    
    private int getMaxNodeWidth() {
        int n = 0;
        for (int i = this.getRowCount() - 1; i >= 0; --i) {
            VariableHeightLayoutCache.TreeStateNode node = this.getNode(i);
            int n2 = node.getPreferredWidth() + node.getXOrigin();
            if (n2 > n) {
                n = n2;
            }
        }
        return n;
    }
    
    private VariableHeightLayoutCache.TreeStateNode createNodeForValue(Object o) {
        return new VariableHeightLayoutCache.TreeStateNode(o);
    }
    
    static void access$000(VariableHeightLayoutCache variableHeightLayoutCache, VariableHeightLayoutCache.TreeStateNode treeStateNode) {
        variableHeightLayoutCache.addMapping(treeStateNode);
    }
    
    static void access$300(VariableHeightLayoutCache variableHeightLayoutCache, VariableHeightLayoutCache.TreeStateNode treeStateNode) {
        variableHeightLayoutCache.removeMapping(treeStateNode);
    }
    
    static VariableHeightLayoutCache.TreeStateNode access$600(VariableHeightLayoutCache variableHeightLayoutCache, Object o) {
        return variableHeightLayoutCache.createNodeForValue(o);
    }
    
    static void access$700(VariableHeightLayoutCache variableHeightLayoutCache) {
        variableHeightLayoutCache.visibleNodesChanged();
    }
    
    private class TreeStateNode extends DefaultMutableTreeNode
    {
        protected int preferredWidth;
        protected int preferredHeight;
        protected int xOrigin;
        protected int yOrigin;
        protected boolean expanded;
        protected boolean hasBeenExpanded;
        protected TreePath path;
        
        public TreeStateNode(Object o) {
            super(o);
        }
        
        @Override
        public void setParent(MutableTreeNode parent) {
            super.setParent(parent);
            if (parent != null) {
                this.path = ((VariableHeightLayoutCache.TreeStateNode)parent).getTreePath().pathByAddingChild(this.getUserObject());
                VariableHeightLayoutCache.access$000(VariableHeightLayoutCache.this, this);
            }
        }
        
        @Override
        public void remove(int n) {
            ((VariableHeightLayoutCache.TreeStateNode)this.getChildAt(n)).removeFromMapping();
            super.remove(n);
        }
        
        @Override
        public void setUserObject(Object userObject) {
            super.setUserObject(userObject);
            if (this.path != null) {
                VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)this.getParent();
                if (treeStateNode != null) {
                    this.resetChildrenPaths(treeStateNode.getTreePath());
                }
                else {
                    this.resetChildrenPaths(null);
                }
            }
        }
        
        @Override
        public Enumeration children() {
            if (!this.isExpanded()) {
                return DefaultMutableTreeNode.EMPTY_ENUMERATION;
            }
            return super.children();
        }
        
        @Override
        public boolean isLeaf() {
            return VariableHeightLayoutCache.this.getModel().isLeaf(this.getValue());
        }
        
        public Rectangle getNodeBounds(Rectangle rectangle) {
            if (rectangle == null) {
                rectangle = new Rectangle(this.getXOrigin(), this.getYOrigin(), this.getPreferredWidth(), this.getPreferredHeight());
                return rectangle;
            }
            rectangle.x = this.getXOrigin();
            rectangle.y = this.getYOrigin();
            rectangle.width = this.getPreferredWidth();
            rectangle.height = this.getPreferredHeight();
            return rectangle;
        }
        
        public int getXOrigin() {
            if (!this.hasValidSize()) {
                this.updatePreferredSize(this.getRow());
            }
            return this.xOrigin;
        }
        
        public int getYOrigin() {
            if (VariableHeightLayoutCache.this.isFixedRowHeight()) {
                int row = this.getRow();
                if (row == -1) {
                    return -1;
                }
                return VariableHeightLayoutCache.this.getRowHeight() * row;
            }
            else {
                return this.yOrigin;
            }
        }
        
        public int getPreferredHeight() {
            if (VariableHeightLayoutCache.this.isFixedRowHeight()) {
                return VariableHeightLayoutCache.this.getRowHeight();
            }
            if (!this.hasValidSize()) {
                this.updatePreferredSize(this.getRow());
            }
            return this.preferredHeight;
        }
        
        public int getPreferredWidth() {
            if (!this.hasValidSize()) {
                this.updatePreferredSize(this.getRow());
            }
            return this.preferredWidth;
        }
        
        public boolean hasValidSize() {
            return this.preferredHeight != 0;
        }
        
        public int getRow() {
            return VariableHeightLayoutCache.this.visibleNodes.indexOf(this);
        }
        
        public boolean hasBeenExpanded() {
            return this.hasBeenExpanded;
        }
        
        public boolean isExpanded() {
            return this.expanded;
        }
        
        public VariableHeightLayoutCache.TreeStateNode getLastVisibleNode() {
            while (this.isExpanded() && this.getChildCount() > 0) {
                this = (VariableHeightLayoutCache.TreeStateNode)this.getLastChild();
            }
            return this;
        }
        
        public boolean isVisible() {
            if (this == VariableHeightLayoutCache.this.root) {
                return true;
            }
            VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)this.getParent();
            return treeStateNode != null && treeStateNode.isExpanded() && treeStateNode.isVisible();
        }
        
        public int getModelChildCount() {
            if (this.hasBeenExpanded) {
                return super.getChildCount();
            }
            return VariableHeightLayoutCache.this.getModel().getChildCount(this.getValue());
        }
        
        public int getVisibleChildCount() {
            int n = 0;
            if (this.isExpanded()) {
                int childCount = this.getChildCount();
                n += childCount;
                for (int i = 0; i < childCount; ++i) {
                    n += ((VariableHeightLayoutCache.TreeStateNode)this.getChildAt(i)).getVisibleChildCount();
                }
            }
            return n;
        }
        
        public void toggleExpanded() {
            if (this.isExpanded()) {
                this.collapse();
            }
            else {
                this.expand();
            }
        }
        
        public void makeVisible() {
            VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)this.getParent();
            if (treeStateNode != null) {
                treeStateNode.expandParentAndReceiver();
            }
        }
        
        public void expand() {
            this.expand(true);
        }
        
        public void collapse() {
            this.collapse(true);
        }
        
        public Object getValue() {
            return this.getUserObject();
        }
        
        public TreePath getTreePath() {
            return this.path;
        }
        
        protected void resetChildrenPaths(TreePath treePath) {
            VariableHeightLayoutCache.access$300(VariableHeightLayoutCache.this, this);
            if (treePath == null) {
                this.path = new TreePath(this.getUserObject());
            }
            else {
                this.path = treePath.pathByAddingChild(this.getUserObject());
            }
            VariableHeightLayoutCache.access$000(VariableHeightLayoutCache.this, this);
            for (int i = this.getChildCount() - 1; i >= 0; --i) {
                ((VariableHeightLayoutCache.TreeStateNode)this.getChildAt(i)).resetChildrenPaths(this.path);
            }
        }
        
        protected void setYOrigin(int yOrigin) {
            this.yOrigin = yOrigin;
        }
        
        protected void shiftYOriginBy(int n) {
            this.yOrigin += n;
        }
        
        protected void updatePreferredSize() {
            this.updatePreferredSize(this.getRow());
        }
        
        protected void updatePreferredSize(int n) {
            Rectangle nodeDimensions = VariableHeightLayoutCache.this.getNodeDimensions(this.getUserObject(), n, this.getLevel(), this.isExpanded(), VariableHeightLayoutCache.this.boundsBuffer);
            if (nodeDimensions == null) {
                this.xOrigin = 0;
                int n3 = 0;
                this.preferredHeight = n3;
                this.preferredWidth = n3;
                VariableHeightLayoutCache.this.updateNodeSizes = true;
            }
            else if (nodeDimensions.height == 0) {
                this.xOrigin = 0;
                int n2 = 0;
                this.preferredHeight = n2;
                this.preferredWidth = n2;
                VariableHeightLayoutCache.this.updateNodeSizes = true;
            }
            else {
                this.xOrigin = nodeDimensions.x;
                this.preferredWidth = nodeDimensions.width;
                if (VariableHeightLayoutCache.this.isFixedRowHeight()) {
                    this.preferredHeight = VariableHeightLayoutCache.this.getRowHeight();
                }
                else {
                    this.preferredHeight = nodeDimensions.height;
                }
            }
        }
        
        protected void markSizeInvalid() {
            this.preferredHeight = 0;
        }
        
        protected void deepMarkSizeInvalid() {
            this.markSizeInvalid();
            for (int i = this.getChildCount() - 1; i >= 0; --i) {
                ((VariableHeightLayoutCache.TreeStateNode)this.getChildAt(i)).deepMarkSizeInvalid();
            }
        }
        
        protected Enumeration getLoadedChildren(boolean b) {
            if (!b || this.hasBeenExpanded) {
                return super.children();
            }
            Object value = this.getValue();
            TreeModel model = VariableHeightLayoutCache.this.getModel();
            int childCount = model.getChildCount(value);
            this.hasBeenExpanded = true;
            int row = this.getRow();
            if (row == -1) {
                for (int j = 0; j < childCount; ++j) {
                    VariableHeightLayoutCache.TreeStateNode access$601 = VariableHeightLayoutCache.access$600(VariableHeightLayoutCache.this, model.getChild(value, j));
                    this.add(access$601);
                    access$601.updatePreferredSize(-1);
                }
            }
            else {
                ++row;
                for (int i = 0; i < childCount; ++i) {
                    VariableHeightLayoutCache.TreeStateNode access$600 = VariableHeightLayoutCache.access$600(VariableHeightLayoutCache.this, model.getChild(value, i));
                    this.add(access$600);
                    access$600.updatePreferredSize(row++);
                }
            }
            return super.children();
        }
        
        protected void didAdjustTree() {
        }
        
        protected void expandParentAndReceiver() {
            VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)this.getParent();
            if (treeStateNode != null) {
                treeStateNode.expandParentAndReceiver();
            }
            this.expand();
        }
        
        protected void expand(boolean b) {
            if (!this.isExpanded() && !this.isLeaf()) {
                int fixedRowHeight = VariableHeightLayoutCache.this.isFixedRowHeight();
                int preferredHeight = this.getPreferredHeight();
                int row = this.getRow();
                this.expanded = true;
                this.updatePreferredSize(row);
                if (!this.hasBeenExpanded) {
                    Object value = this.getValue();
                    TreeModel model = VariableHeightLayoutCache.this.getModel();
                    int childCount = model.getChildCount(value);
                    this.hasBeenExpanded = true;
                    if (row == -1) {
                        for (int j = 0; j < childCount; ++j) {
                            VariableHeightLayoutCache.TreeStateNode access$601 = VariableHeightLayoutCache.access$600(VariableHeightLayoutCache.this, model.getChild(value, j));
                            this.add(access$601);
                            access$601.updatePreferredSize(-1);
                        }
                    }
                    else {
                        int n = row + 1;
                        for (int i = 0; i < childCount; ++i) {
                            VariableHeightLayoutCache.TreeStateNode access$600 = VariableHeightLayoutCache.access$600(VariableHeightLayoutCache.this, model.getChild(value, i));
                            this.add(access$600);
                            access$600.updatePreferredSize(n);
                        }
                    }
                }
                Enumeration<Object> preorderEnumeration = this.preorderEnumeration();
                preorderEnumeration.nextElement();
                int yOrigin;
                if (!(fixedRowHeight == 0)) {
                    yOrigin = 0;
                }
                else if (this == VariableHeightLayoutCache.this.root && !VariableHeightLayoutCache.this.isRootVisible()) {
                    yOrigin = 0;
                }
                else {
                    yOrigin = this.getYOrigin() + this.getPreferredHeight();
                }
                if (fixedRowHeight == 0) {
                    while (preorderEnumeration.hasMoreElements()) {
                        VariableHeightLayoutCache.TreeStateNode obj = (VariableHeightLayoutCache.TreeStateNode)preorderEnumeration.nextElement();
                        if (!VariableHeightLayoutCache.this.updateNodeSizes && !obj.hasValidSize()) {
                            obj.updatePreferredSize(row + 1);
                        }
                        obj.setYOrigin(yOrigin);
                        yOrigin += obj.getPreferredHeight();
                        VariableHeightLayoutCache.this.visibleNodes.insertElementAt(obj, ++row);
                    }
                }
                else {
                    while (preorderEnumeration.hasMoreElements()) {
                        VariableHeightLayoutCache.this.visibleNodes.insertElementAt((Object)preorderEnumeration.nextElement(), ++row);
                    }
                }
                if (b && (row != row || this.getPreferredHeight() != preferredHeight)) {
                    if (fixedRowHeight == 0 && ++row < VariableHeightLayoutCache.this.getRowCount()) {
                        int n2 = yOrigin - (this.getYOrigin() + this.getPreferredHeight()) + (this.getPreferredHeight() - preferredHeight);
                        for (int k = VariableHeightLayoutCache.this.visibleNodes.size() - 1; k >= row; --k) {
                            ((VariableHeightLayoutCache.TreeStateNode)VariableHeightLayoutCache.this.visibleNodes.elementAt(k)).shiftYOriginBy(n2);
                        }
                    }
                    this.didAdjustTree();
                    VariableHeightLayoutCache.access$700(VariableHeightLayoutCache.this);
                }
                if (VariableHeightLayoutCache.this.treeSelectionModel != null) {
                    VariableHeightLayoutCache.this.treeSelectionModel.resetRowSelection();
                }
            }
        }
        
        protected void collapse(boolean b) {
            if (this.isExpanded()) {
                Enumeration<Object> preorderEnumeration = this.preorderEnumeration();
                preorderEnumeration.nextElement();
                int n = 0;
                int fixedRowHeight = VariableHeightLayoutCache.this.isFixedRowHeight();
                int n2;
                if (!(fixedRowHeight == 0)) {
                    n2 = 0;
                }
                else {
                    n2 = this.getPreferredHeight() + this.getYOrigin();
                }
                int preferredHeight = this.getPreferredHeight();
                int row = this.getRow();
                if (fixedRowHeight == 0) {
                    while (preorderEnumeration.hasMoreElements()) {
                        VariableHeightLayoutCache.TreeStateNode treeStateNode = (VariableHeightLayoutCache.TreeStateNode)preorderEnumeration.nextElement();
                        if (treeStateNode.isVisible()) {
                            ++n;
                            n2 = treeStateNode.getYOrigin() + treeStateNode.getPreferredHeight();
                        }
                        else {
                            continue;
                        }
                    }
                }
                else {
                    while (preorderEnumeration.hasMoreElements()) {
                        if (((VariableHeightLayoutCache.TreeStateNode)preorderEnumeration.nextElement()).isVisible()) {
                            ++n;
                        }
                        else {
                            continue;
                        }
                    }
                }
                for (int i = n + row; i > row; --i) {
                    VariableHeightLayoutCache.this.visibleNodes.removeElementAt(i);
                }
                this.expanded = false;
                if (row == -1) {
                    this.markSizeInvalid();
                }
                else if (b) {
                    this.updatePreferredSize(row);
                }
                if (row != -1 && b && (n > 0 || preferredHeight != this.getPreferredHeight())) {
                    int n3 = n2 + (this.getPreferredHeight() - preferredHeight);
                    if (fixedRowHeight == 0 && row + 1 < VariableHeightLayoutCache.this.getRowCount() && n3 != n2) {
                        int n4 = n3 - n2;
                        for (int j = row + 1; j < VariableHeightLayoutCache.this.visibleNodes.size(); ++j) {
                            ((VariableHeightLayoutCache.TreeStateNode)VariableHeightLayoutCache.this.visibleNodes.elementAt(j)).shiftYOriginBy(n4);
                        }
                    }
                    this.didAdjustTree();
                    VariableHeightLayoutCache.access$700(VariableHeightLayoutCache.this);
                }
                if (VariableHeightLayoutCache.this.treeSelectionModel != null && n > 0 && row != -1) {
                    VariableHeightLayoutCache.this.treeSelectionModel.resetRowSelection();
                }
            }
        }
        
        protected void removeFromMapping() {
            if (this.path != null) {
                VariableHeightLayoutCache.access$300(VariableHeightLayoutCache.this, this);
                for (int i = this.getChildCount() - 1; i >= 0; --i) {
                    ((VariableHeightLayoutCache.TreeStateNode)this.getChildAt(i)).removeFromMapping();
                }
            }
        }
    }
    
    private class VisibleTreeStateNodeEnumeration implements Enumeration<TreePath>
    {
        protected VariableHeightLayoutCache.TreeStateNode parent;
        protected int nextIndex;
        protected int childCount;
        
        protected VisibleTreeStateNodeEnumeration(VariableHeightLayoutCache variableHeightLayoutCache, VariableHeightLayoutCache.TreeStateNode treeStateNode) {
            this(variableHeightLayoutCache, treeStateNode, -1);
        }
        
        protected VisibleTreeStateNodeEnumeration(VariableHeightLayoutCache.TreeStateNode parent, int nextIndex) {
            this.parent = parent;
            this.nextIndex = nextIndex;
            this.childCount = this.parent.getChildCount();
        }
        
        @Override
        public boolean hasMoreElements() {
            return this.parent != null;
        }
        
        @Override
        public TreePath nextElement() {
            if (!this.hasMoreElements()) {
                throw new NoSuchElementException("No more visible paths");
            }
            TreePath treePath;
            if (this.nextIndex == -1) {
                treePath = this.parent.getTreePath();
            }
            else {
                treePath = ((VariableHeightLayoutCache.TreeStateNode)this.parent.getChildAt(this.nextIndex)).getTreePath();
            }
            this.updateNextObject();
            return treePath;
        }
        
        protected void updateNextObject() {
            if (!this.updateNextIndex()) {
                this.findNextValidParent();
            }
        }
        
        protected boolean findNextValidParent() {
            if (this.parent == VariableHeightLayoutCache.this.root) {
                this.parent = null;
                return false;
            }
            while (this.parent != null) {
                VariableHeightLayoutCache.TreeStateNode parent = (VariableHeightLayoutCache.TreeStateNode)this.parent.getParent();
                if (parent != null) {
                    this.nextIndex = parent.getIndex(this.parent);
                    this.parent = parent;
                    this.childCount = this.parent.getChildCount();
                    if (this.updateNextIndex()) {
                        return true;
                    }
                    continue;
                }
                else {
                    this.parent = null;
                }
            }
            return false;
        }
        
        protected boolean updateNextIndex() {
            if (this.nextIndex == -1 && !this.parent.isExpanded()) {
                return false;
            }
            if (this.childCount == 0) {
                return false;
            }
            if (++this.nextIndex >= this.childCount) {
                return false;
            }
            VariableHeightLayoutCache.TreeStateNode parent = (VariableHeightLayoutCache.TreeStateNode)this.parent.getChildAt(this.nextIndex);
            if (parent != null && parent.isExpanded()) {
                this.parent = parent;
                this.nextIndex = -1;
                this.childCount = parent.getChildCount();
            }
            return true;
        }
    }
}
